import React from 'react';
import { render, fireEvent, screen, act } from '@/test-utils/index';
// import { act } from 'react-dom/test-utils';
import NsAlert from './NsAlert';

const addEventListenerMock = jest.spyOn(window, 'addEventListener');
const removeEventListenerMock = jest.spyOn(window, 'removeEventListener');

describe('NsAlert', () => {
    test('renders alert with provided props', () => {
        render(
            <NsAlert
                context="error"
                title="Error Alert"
                threshhold={800}
                type="banner"
                visible
                head={<span>Alert Head</span>}
                body={<p>Alert Body</p>}
                foot={<span>Dismiss</span>}
                onDismiss={() => {}}
            />
        );

        expect(screen.getByText('Error Alert')).toBeInTheDocument();
        expect(screen.getByText('Alert Head')).toBeInTheDocument();
        expect(screen.getByText('Alert Body')).toBeInTheDocument();
        expect(screen.getByText('Dismiss')).toBeInTheDocument();
    });

    test('calls onDismiss function when dismissed', () => {
        const onDismissMock = jest.fn();
        render(
            <NsAlert
                context="info"
                title="Info Alert"
                threshhold={600}
                type="auto"
                visible
                head={<span>Alert Head</span>}
                body={<p>Alert Body</p>}
                onDismiss={onDismissMock}
            />
        );

        expect(screen.getByTestId('alert-close-button')).toBeInTheDocument();
        const button = screen.getByTestId('alert-close-button');
        fireEvent.click(button);
        expect(onDismissMock).toHaveBeenCalled();
    });

    test('hides alert when not visible', () => {
        render(
            <NsAlert
                context="success"
                title="Success Alert"
                threshhold={700}
                type="card"
                visible={false}
                head={<span>Alert Head</span>}
                body={<p>Alert Body</p>}
            />
        );

        expect(screen.getByTestId('alert-container')).toHaveStyle('display: none');
    });

    test('adds and removes event listener on mount/unmount', () => {
        act(() => {
            render(
                <NsAlert
                    context="notice"
                    title="Success Alert"
                    threshhold={700}
                    type="card"
                    visible={false}
                    head={<span>Alert Head</span>}
                    body={<p>Alert Body</p>}
                />
            );
        });

        expect(addEventListenerMock).toHaveBeenCalledWith('resize', expect.any(Function));

        expect(removeEventListenerMock).toHaveBeenCalledWith('resize', expect.any(Function));
    });

    test('updates windowWidth on resize', () => {
        const originalInnerWidth = window.innerWidth;
        window.innerWidth = 500; // Set a desired window width for testing

        act(() => {
            render(<NsAlert />);
        });

        // Simulate a resize event by restoring the original innerWidth
        window.innerWidth = originalInnerWidth;

        act(() => {
            window.dispatchEvent(new Event('resize'));
        });
    });

    test('get props error in console for types', () => {
        render(
            <NsAlert
                context="warning"
                title="warning Alert"
                threshhold={800}
                type="noCard"
                visible
                head={<span>Alert Head</span>}
                body={<p>Alert Body</p>}
            />
        );

        expect(screen.getByTestId('alert-container')).toHaveStyle('display: block');
    });

    test('get props error in console for context', () => {
        render(
            <NsAlert
                context="warningError"
                title="warning Alert"
                threshhold={800}
                type="card"
                visible
                head={<span>Alert Head</span>}
                body={<p>Alert Body</p>}
            />
        );

        expect(screen.getByTestId('alert-container')).toHaveStyle('display: block');
    });
});
