/* eslint-disable react/require-default-props */
import React, { useEffect, useState } from 'react';
import {
    AlertContainer,
    Alert,
    AlertIcon,
    AlertHead,
    AlertCloseButton,
    AlertContent,
    AlertFoot,
    AlertBody,
} from './NsAlert.styled';
import type { NsAlertProps, HeaderProps, ContextType, AlertType } from './types.d';
import { NsIcon } from '../icon';
import { NsIconNameType } from '../icon/types';
import { FoundationTheme } from '@nuskin/foundation-theme/src/themes';

const Header: React.FC<HeaderProps> = ({ children, head }) => (
    <>
        {children}
        {head}
    </>
);

function isValidContextValue(value: string): boolean {
    return (value as ContextType) !== undefined;
}

function isValidTypeValue(value: string): boolean {
    return (value as AlertType) !== undefined;
}

const colorMap: { [key: string]: FoundationTheme.PaletteColors } = {
    error: 'error',
    notice: 'info',
    success: 'secondary',
    warning: 'warning',
    default: 'info',
};

const iconMap: { [key: string]: NsIconNameType } = {
    error: 'alertError',
    success: 'success',
    warning: 'warning',
    default: 'infoFilled',
};

export default function NsAlert({
    context = 'info',
    title = 'Information',
    threshhold = 600,
    type = 'banner',
    visible = false,
    head,
    body,
    foot,
    onDismiss = undefined,
}: NsAlertProps) {
    const [alertType, setAlertType] = useState('banner');
    const [windowWidth, setWindowWidth] = useState(window.innerWidth);

    const hasDismissListener = typeof onDismiss === 'function';

    const getColor = () => {
        return colorMap[context] || colorMap.default;
    };

    const getIcon = () => {
        return iconMap[context] || iconMap.default;
    };

    const getTitleText = () => {
        if (title) {
            return title;
        }
    };

    const handleDismiss = () => {
        if (hasDismissListener) {
            onDismiss();
        }
    };

    const handleResize = () => {
        setWindowWidth(window.innerWidth);
    };

    useEffect(() => {
        let aType = '';
        if (type === 'auto') {
            aType = windowWidth < threshhold ? 'card' : 'banner';
        } else {
            aType = type;
        }
        setAlertType(aType);
    }, [threshhold, type, windowWidth]);

    useEffect(() => {
        if (!isValidContextValue(context)) {
            console.error(`Invalid prop value for 'context': ${context}`);
        }

        if (!isValidTypeValue(type)) {
            console.error(`Invalid prop value for 'type': ${type}`);
        }
    }, [context, type]);

    useEffect(() => {
        window.addEventListener('resize', handleResize);
        return () => {
            window.removeEventListener('resize', handleResize);
        };
    }, []);

    return (
        <AlertContainer visible={visible} data-testid="alert-container">
            <Alert type={alertType as AlertType} context={context as ContextType}>
                <AlertIcon>
                    <NsIcon name={getIcon()} color={getColor()} size="large" />
                </AlertIcon>
                <AlertHead>
                    <Header head={<span>{getTitleText()}</span>}>{head}</Header>
                </AlertHead>
                {hasDismissListener ? (
                    <AlertCloseButton data-testid="alert-close-button" onClick={handleDismiss}>
                        <NsIcon name="close" size="small" />
                    </AlertCloseButton>
                ) : null}
                <AlertContent type={alertType as AlertType}>
                    <AlertBody>{body}</AlertBody>
                    {foot ? <AlertFoot type={alertType as AlertType}>{foot}</AlertFoot> : null}
                </AlertContent>
            </Alert>
        </AlertContainer>
    );
}
