import React, { useState, useRef, useEffect } from 'react';
import { SelectContainer, SelectButton, SelectList, SelectHead, SelectLabel } from './NsSelect.styled';
import { NsIcon } from '../icon';
import { SelectProps } from './types.d';

const NsSelect: React.FC<SelectProps> = ({
    selectedValue,
    selectedLabel,
    onSelect,
    children,
    disabled = false,
    type = 'outlined',
    placeholder = '',
    selectBorderColor = 'accentColorB',
    closeOnMouseLeave = false,
    showListDivider = true,
    label,
    readOnly = false,
    errorMessage,
}) => {
    const [isOpen, setIsOpen] = useState(false);
    const [value, setValue] = useState(selectedValue || '');
    const selectContainerRef = useRef<HTMLDivElement>(null);
    const selectHeadRef = useRef<HTMLDivElement>(null);

    const closeSelect = () => {
        if (closeOnMouseLeave) {
            setIsOpen(false);
        }
    };

    const handleSelect = (input: string) => {
        onSelect(input);
        setIsOpen(false);
        setValue(input);
    };

    // eslint-disable-next-line react-hooks/exhaustive-deps
    const handleClickOutside = (event: MouseEvent) => {
        if (
            selectContainerRef.current &&
            !selectContainerRef.current.contains(event.target as Node) &&
            isOpen
        ) {
            setIsOpen(false);
        }
    };

    const handleMouseClick = () => {
        if (disabled) return;
        setIsOpen(!isOpen);
        if (!isOpen) {
            setTimeout(() => {
                selectHeadRef.current?.focus();
            }, 0);
        }
    };

    useEffect(() => {
        document.addEventListener('click', handleClickOutside);
        return () => {
            document.removeEventListener('click', handleClickOutside);
        };
    }, [handleClickOutside, isOpen]);

    return (
        <SelectContainer
            data-testid="select-container"
            disabled={disabled || readOnly}
            ref={selectContainerRef}
            style={{
                borderColor: errorMessage ? 'red' : undefined,
                color: errorMessage ? 'red' : 'inherit',
            }}
        >
            {label && <SelectLabel style={{ color: errorMessage ? 'red' : 'inherit' }}>{label}</SelectLabel>}
            <SelectHead
                data-testid="select-head"
                disabled={disabled || readOnly}
                type={type}
                onMouseDown={handleMouseClick}
                tabIndex={0}
                ref={selectHeadRef}
            >
                <SelectButton>{selectedLabel || placeholder}</SelectButton>
                {!readOnly && (
                    <NsIcon testId="chevron-icon" name={isOpen ? 'arrowUp' : 'arrowDown'} color="primary" />
                )}
            </SelectHead>
            {isOpen ? (
                <SelectList
                    data-testid="select-list"
                    onMouseLeave={closeSelect}
                    borderColor={selectBorderColor}
                    className='selectList'
                >
                    <div style={{ maxHeight: '200px', overflowY: 'auto' }}>
                        {React.Children.map(children, (child) => {
                            if (React.isValidElement(child)) {
                                return React.cloneElement(child, {
                                    // @ts-ignore
                                    onSelect: handleSelect,
                                    selectedValue: value,
                                    showListDivider,
                                });
                            }
                            return null;
                        })}
                    </div>
                </SelectList>
            ) : null}
            {errorMessage && <div style={{ color: 'red' }}>{errorMessage}</div>}
        </SelectContainer>
    );
};

export default NsSelect;
